// communication server	(c) mg sai uni ulm january 2003
#include <qapplication.h>
#include <qstring.h>
#include <qtextstream.h>
#include <qsocket.h>
#include <qserversocket.h>
#include <qnetwork.h>
#include <qfile.h>
#include <unistd.h>
#include "server.h"


// called when a new client connects to server
void server::newConnection( int socketId )
{
  QSocket *nsocket = new QSocket( this );	// create empty Qsocket
  nsocket->setSocket( socketId );	// connect QSocket to filedescriptor
  // slot called when new data arrives
  QObject::connect( nsocket, SIGNAL( readyRead() ), this, SLOT( slotIncomingData() ) );
  // slot called when client closes the socket
  QObject::connect( nsocket, SIGNAL( connectionClosed() ),
					   this, SLOT( lostConnection() ) );
  // slot called when socket has written all data
  QObject::connect( nsocket, SIGNAL( delayedCloseFinished() ),
					   this, SLOT( slotDeleteSocket() ) );

  sendUserList(nsocket);	// send current user list to new client
}

// send user list to a client identified by QSocket
void server::sendUserList(QSocket *s) {
	QTextStream stream( s);
	stream <<"LIST:";
	std::map<QString, QSocket*>::iterator i = user.begin();
	for ( ; i!=user.end(); i++)  
		stream <<i->first<<":";
	stream <<"\n";
}

// This slot is called when data arrives via one of the data sockets
void server::slotIncomingData()
{
  QSocket *socket = (QSocket*)sender();	// determine sending socket

  while( socket->canReadLine() ) {	// line already complete?
	QTextStream stream( socket );
	QString line = socket->readLine();	// get line
	line = line.left(line.length()-1);	// delete cr
	if (line.contains("\r")>0) 	// linefeed?
		line = line.left(line.length()-1);	// delete lf too
		
	// debugging info to console
	QFile fp;
	fp.open(IO_WriteOnly, 1);
	QTextStream term(&fp);
	term <<line<<endl;

	if (line.contains("LIST!")>0) {	// LIST-command
		sendUserList(socket);
	} else if (line.contains("USER!")>0) {	// new user name
		user[line.right(line.length()-5)] = socket;
		// send new user list to all clients!
		std::map<QString, QSocket*>::iterator i = user.begin();
		for ( ; i!=user.end(); i++)  
			sendUserList(i->second);
	} else if (line.contains("SEND!")>0) {	// send text to a user
		QString hlp = line.right(line.length()-5);
		int pos = hlp.find('!');
		if (!pos)		// incomplete command
			return;

		QString mesg = hlp.right(hlp.length()-pos-1);	// message
		QString recv = hlp.left(pos);	// to whom to send to

		QSocket *recvSocket=0;	// find corresponding socket
		QString sender = "XXX";	// sender's name
		std::map<QString, QSocket*>::iterator i = user.begin();
		for ( ; i!=user.end(); i++)  {
			if (i->first==recv)	// find receiver socket
				recvSocket = i->second;
			if (i->second == socket) // find sender's name
				sender = i->first;
		}
		if (recvSocket) {
			QTextStream recp( recvSocket );
			recp <<sender<<": "<<mesg<<endl;
		}
		else 
			stream <<"ERROR: recipient not found!!\n";
	}
	else if (line.contains("QUIT!")>0) {	// quit command
		std::map<QString, QSocket*>::iterator i = user.begin();
		for ( ; i!=user.end(); i++)  {
			if (i->second == socket) // find sender's map entry
				break;
		}
		if (i!=user.end()) {
			user.erase(i);	// delete sender of quit command
			i = user.begin();
			for ( ; i!=user.end(); i++)  
				sendUserList(i->second);
		}
	}
  }
}


/* This slot is called after the socket has been closed. */
void server::slotDeleteSocket()
{
  // There could be more than one socket, so we need to find out who
  // sent the signal.
  QSocket *socket = (QSocket*)sender();
  delete socket;
  socket = NULL;
}

// called when a client closed the connection

void server::lostConnection() {
	QSocket *socket = (QSocket*)sender();
	socket->close();
	delete socket;
}

int main( int argc, char* argv[] )
{
  // We need a QApplication object, but there won't be any widgets.
  QApplication a( argc, argv, false );	// we don't need no GUI = "false"

  // Creating a server object starts the server.
  server comserver;
  if (comserver.ok())		// should be ok - if port not already in use !
	return a.exec();
  else
	return 1;
}
